VERSION 5.00
Begin VB.UserControl ezToolbar 
   Alignable       =   -1  'True
   ClientHeight    =   660
   ClientLeft      =   0
   ClientTop       =   0
   ClientWidth     =   4800
   ScaleHeight     =   44
   ScaleMode       =   3  'Pixel
   ScaleWidth      =   320
End
Attribute VB_Name = "ezToolbar"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
'   ezToolbar   - Gives a Office 97 Style Toolbar Control
'   Author      - Joseph J Guadagno
'   Date        - 6/27/97
' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#

Option Explicit

' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
' Types
' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
Private Type RECT
    Left As Long
    Top As Long
    Right As Long
    Bottom As Long
End Type

Private Type POINTAPI
    x As Long
    y As Long
End Type

' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
' Declarations
' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
Private Declare Function DrawEdge Lib "user32" (ByVal hdc As Long, _
    qrc As RECT, ByVal edge As Long, ByVal grfFlags As Long) As Long

Private Declare Function GetCursorPos Lib "user32" (lpPoint As POINTAPI) _
    As Long
Private Declare Function InvertRect Lib "user32" (ByVal hdc As Long, _
    lpRect As RECT) As Long
Private Declare Function GetWindowDC Lib "user32" (ByVal hwnd As Long) As Long
Private Declare Function GetDesktopWindow Lib "user32" () As Long
Private Declare Function SetParent Lib "user32" (ByVal hWndChild As Long, ByVal hWndNewParent As Long) As Long
Private Declare Function GetSystemMetrics Lib "user32" (ByVal nIndex As Long) _
    As Long


' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
' Constants
' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
'  Draw Edge Constants
Private Const BDR_RAISEDINNER = &H4
Private Const BDR_RAISEDOUTER = &H1
Private Const BDR_SUNKENINNER = &H8
Private Const BDR_SUNKENOUTER = &H2
Private Const BF_BOTTOM = &H8
Private Const BF_LEFT = &H1
Private Const BF_RIGHT = &H4
Private Const BF_TOP = &H2
Private Const BF_SOFT = &H1000
Private Const BF_RECT = (BF_LEFT Or BF_TOP Or BF_RIGHT Or BF_BOTTOM)
Private Const EDGE_BUMP = (BDR_RAISEDOUTER Or BDR_SUNKENINNER)
Private Const EDGE_ETCHED = (BDR_SUNKENOUTER Or BDR_RAISEDINNER)
Private Const EDGE_RAISED = (BDR_RAISEDOUTER Or BDR_RAISEDINNER)
Private Const EDGE_SUNKEN = (BDR_SUNKENOUTER Or BDR_SUNKENINNER)

' System Metrics Constants
Private Const SM_CXBORDER = 5
Private Const SM_CYBORDER = 6
Private Const SM_CXDLGFRAME = 7
Private Const SM_CYCAPTION = 4
Private Const SM_CYDLGFRAME = 8

' Appearance Constants
Private Const iAppearanceFlat = 0
Private Const iAppearance3D = 1

' Draw Contants
Private Const ezHorizontal = True
Private Const ezVertical = False

' Size Constants
Private Const MAX_WIDTH = 405
Private Const MAX_HEIGHT = 405
Private Const lCboButtonSize = 8


' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
'   Property Defaults
' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
'Default Property Values:
Const m_def_MaskColor = &HC0C0C0


' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
'   Property Variables
' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
'Property Variables:
Dim m_MaskColor As OLE_COLOR


' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
'   Events
' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
'Event Declarations:
Event Click(ButtonIndex As Long, ButtonKey As Variant)
Event MouseDown(Button As Integer, Shift As Integer, x As Single, y As Single) 'MappingInfo=UserControl,UserControl,-1,MouseDown
Event MouseMove(Button As Integer, Shift As Integer, x As Single, y As Single) 'MappingInfo=UserControl,UserControl,-1,MouseMove
Event MouseUp(Button As Integer, Shift As Integer, x As Single, y As Single) 'MappingInfo=UserControl,UserControl,-1,MouseUp

' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
'   Public Exposed Collections/Classes
' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#

' Expose the Buttons Collection and Button Class
Public Buttons As New clsButtons
Public Button As New clsButton

' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
'   Private Collections and Class
' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
' The Paint Effect Class For Drawing
Private cPaint As PaintEffects

Private Enum ezFrameType    ' Enum for type of frames to draw
    ezFrameNone = 0
    ezFrameRaised
    ezFrameSunken
End Enum

' Enum for the portion Identifier for a combo box
Private Enum ezPortionType
    ezPortionImage = 0
    ezPortionButton
End Enum

' The Mouse is down on the User Control
Private bMouseDown      As Boolean
' The Button that the mouse was down on originally
Private lMouseDownOn    As Long

' The pos of the cursor with respect to the toolbar
Private mlngXOffset     As Long
Private mlngYOffset     As Long

' HOlders for the amount of Twips in a pixel
Private mlngTwipsX      As Long
Private mlngTwipsY      As Long

' The screen device context
Private lngScreenDC     As Long
' The type for drawing the inverted rectangle when the toolbar is being dragged
Private udtRect         As RECT     ' The rect for the inverted rectangle
Private udtInnerRect    As RECT     ' The rect for the inside of the inverted

' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
'  ButtonAtLocation     - Determines what button is beneath the
'       mousepointer
'   Author      - Joseph J Guadagno
' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
Public Function ButtonAtLocation(x As Single, y As Single) As Long
Attribute ButtonAtLocation.VB_Description = "Returns the Button that is underneath the mouse at x,y"
    Dim iTempIdx As Integer
    
    ButtonAtLocation = 0
    For iTempIdx = 1 To Buttons.Count
        If x >= Buttons.Item(iTempIdx).Left And _
            x <= (Buttons.Item(iTempIdx).Left + Buttons.Item(iTempIdx).Width) And _
            y >= Buttons.Item(iTempIdx).Top And _
            y <= (Buttons.Item(iTempIdx).Top + Buttons.Item(iTempIdx).Height) Then
            
            ' We have a Match
            ButtonAtLocation = iTempIdx
            Exit For
        End If
    Next iTempIdx

End Function

' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
'  DrawFrame    - Draws a 3D frame around the button
'   Parameters  - lButton, the button to draw the frame around
'               - lFrameStyle, the type of frame to draw
'   Author      - Joseph J Guadagno
' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#

Private Sub DrawFrame(lButton As Long, lFrameStyle As ezFrameType, _
                        Optional lPortion As ezPortionType)
    
    ' Declare the dimensions
    Dim x1 As Long
    Dim x2 As Long
    Dim y1 As Long
    Dim y2 As Long
    
    ' If the Style is a combo Box , we need to draw only a certain
    ' portion of it
    If Buttons.Item(lButton).Style = ezComboBox Then
        ' Create the Dimensions for the specified portion
        Select Case lPortion
            Case ezPortionImage
                With Buttons.Item(lButton)
                    x1 = .Left - 3
                    x2 = .Left + .Width - lCboButtonSize + 1
                    y1 = .Top - 3
                    y2 = .Top + .Height + 1
                End With
                
            Case ezPortionButton
                With Buttons.Item(lButton)
                    x1 = .Left + .Width - lCboButtonSize + 1
                    x2 = .Left + .Width + 1
                    y1 = .Top - 3
                    y2 = .Top + .Height + 1
                End With
        End Select
    
    Else
        With Buttons.Item(lButton)
            x1 = .Left - 3
            x2 = .Left + .Width + 1
            y1 = .Top - 3
            y2 = .Top + .Height + 1
        End With
    End If

    Select Case lFrameStyle
        Case ezFrameNone
             ' Draw the Line
            Line (x1, y1)-(x1, y2), vbButtonFace    ' Top
            Line (x1, y1)-(x2, y1), vbButtonFace    ' Left
            Line (x2, y1)-(x2, y2), vbButtonFace    ' Right
            Line (x1, y2)-(x2 + 1, y2), vbButtonFace    ' Bottom
        Case ezFrameRaised
            ' Draw the Line
            Line (x1, y1)-(x1, y2), vb3DHighlight   ' Top
            Line (x1, y1)-(x2, y1), vb3DHighlight   ' Left
            Line (x2, y1)-(x2, y2), vb3DShadow      ' Right
            Line (x1, y2)-(x2 + 1, y2), vb3DShadow      ' Bottom
        Case ezFrameSunken
            ' Draw the Line
            Line (x1, y1)-(x1, y2), vb3DShadow      ' Top
            Line (x1, y1)-(x2, y1), vb3DShadow      ' Left
            Line (x2, y1)-(x2, y2), vb3DHighlight   ' Right
            Line (x1, y2)-(x2 + 1, y2), vb3DHighlight   ' Bottom
    End Select
    
    
DrawFrame_ComboBox:

    
    
End Sub

' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
'   Draws a 3d Line
'
'   Parameters  : lStartLocation , the location to begin the line
'               : bHorizontal, a flag to specify to draw the line
'                       horizontal or vertical
'               : lLineType,  the type of line to , raised, sunken or None
'
' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#

Private Sub DrawLine(lStartLocation As Long, Optional bHorizontal As Boolean = ezHorizontal, Optional lLineType As ezFrameType = ezFrameRaised)
    
    Select Case bHorizontal
        ' Horizontal Line
        Case ezHorizontal
            Select Case lLineType
                Case ezFrameRaised

                    UserControl.Line (lStartLocation, 3)-(lStartLocation, UserControl.ScaleHeight - 3), vb3DHighlight
                    UserControl.Line (lStartLocation + 1, 3)-(lStartLocation + 1, UserControl.ScaleHeight - 3), vb3DShadow
                Case ezFrameSunken
                    UserControl.Line (lStartLocation, 3)-(lStartLocation, UserControl.ScaleHeight - 3), vb3DShadow
                    UserControl.Line (lStartLocation + 1, 3)-(lStartLocation + 1, UserControl.ScaleHeight - 3), vb3DHighlight
                Case ezFrameNone
                    UserControl.Line (lStartLocation, 3)-(lStartLocation, UserControl.ScaleHeight - 3), vbButtonFace
                    UserControl.Line (lStartLocation + 1, 3)-(lStartLocation + 1, UserControl.ScaleHeight - 3), vbButtonFace
            
            End Select
        ' Vertical Line
        Case ezVertical
            Select Case lLineType
                Case ezFrameRaised
                    ' Raised Line
                    UserControl.Line (3, lStartLocation)-(UserControl.ScaleWidth - 3, lStartLocation), vb3DHighlight
                    UserControl.Line (3, lStartLocation + 1)-(UserControl.ScaleWidth - 3, lStartLocation + 1), vb3DShadow
                Case ezFrameSunken
                    ' Sunken Line
                    UserControl.Line (3, lStartLocation)-(UserControl.ScaleWidth - 3, lStartLocation), vb3DShadow
                    UserControl.Line (3, lStartLocation + 1)-(UserControl.ScaleWidth - 3, lStartLocation + 1), vb3DHighlight
                Case ezFrameNone
                    UserControl.Line (3, lStartLocation)-(UserControl.ScaleWidth - 3, lStartLocation), vbButtonFace
                    UserControl.Line (3, lStartLocation + 1)-(UserControl.ScaleWidth - 3, lStartLocation + 1), vbButtonFace
            
            End Select
        
    End Select
End Sub

' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
'  DrawToolbar  - Does the actual painting of the images on the
'                 Toolbar
'   Author      - Joseph J Guadagno
' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
Private Sub DrawToolbar()
    
    ' #%%%%%%%%%%%%%%%%%%%%%%%%%#
    '  Dim all of our variables
    ' #%%%%%%%%%%%%%%%%%%%%%%%%%#
    Dim lPosX As Long           ' Current X Position to Draw
    Dim lPosY As Long           ' Current Y Position To Draw
                                
    Dim lPosXcbo As Long        ' Current X Position to Draw cboButton
    Dim lPosYcbo As Long        ' Current Y Position To Draw cboButton
                                
    Dim lBtnHeight As Long      ' The Height of the Button To Draw
    Dim lBtnWidth As Long       ' The Width of the button to Draw
    
    Dim EdgeRC As RECT          ' Rect for DRawEdge
    
    ' Miscellanous Variables
    Dim lReturn As Long         ' Return Variables from the API Calls
    Dim iCurrentButton As Long  ' Counter for the Buttons
    
    ' empty the control
    UserControl.Cls
    
    ' #%%%%%%%%%%%%%%%%%%%%%%%%%#
    ' Create the Initial Surrounding Border and Gripper Lines
    ' #%%%%%%%%%%%%%%%%%%%%%%%%%#
    
    ' Draw the border of the Control
    If UserControl.Extender.Appearance = iAppearance3D Then
    
        With EdgeRC
            .Top = 0
            .Left = 0
            .Right = UserControl.ScaleWidth
            .Bottom = UserControl.ScaleHeight
        End With
        DrawEdge UserControl.hdc, EdgeRC, BDR_RAISEDOUTER, BF_RECT Or BF_SOFT
    End If
    
    Select Case UserControl.Extender.Align
        Case vbAlignNone, vbAlignTop, vbAlignBottom
            DrawLine 4, ezHorizontal, ezFrameRaised
            DrawLine 7, ezHorizontal, ezFrameRaised
        
        Case vbAlignLeft, vbAlignRight
            DrawLine 4, ezVertical, ezFrameRaised
            DrawLine 7, ezVertical, ezFrameRaised

    End Select
    
    ' Create the PaintEffects Reference
    Set cPaint = New PaintEffects
        
    ' Set the Left and Top Position for Drawing
    Select Case UserControl.Extender.Align
        Case vbAlignNone, vbAlignTop, vbAlignBottom
            lPosX = 14
            lPosY = 6
        Case vbAlignLeft, vbAlignRight
            lPosX = 6
            lPosY = 14
    End Select
    
    ' #%%%%%%%%%%%%%%%%%%%%%%%%#
    ' Begin Drawing the Buttons
    ' #%%%%%%%%%%%%%%%%%%%%%%%%#
    
    For iCurrentButton = 1 To Buttons.Count
    
        ' Determine the Height and the Width of the button
        
        ' NOTE : The conversion is neccessary because the conversion will round height and
        '           width out
        With Buttons.Item(iCurrentButton)
            If .Style <> ezSeparator Then
                lBtnHeight = UserControl.ScaleX(.Image.Height, vbHimetric, vbTwips) / mlngTwipsX
                lBtnWidth = UserControl.ScaleY(.Image.Width, vbHimetric, vbTwips) / mlngTwipsY
            End If
            ' Set the Left & Top positions for the Button
            .Left = lPosX
            .Top = lPosY
            ' Set the Hieght & Width of the Button
            .Height = lBtnHeight
            .Width = lBtnWidth
        
            ' Look to see what type of button this is "ezButtonStyle"
            Select Case .Style
                Case ezDefault
                
                    If .Enabled Then
                        cPaint.PaintTransparentStdPic UserControl.hdc, lPosX, lPosY, lBtnWidth, _
                            lBtnHeight, .Image, 0, 0, m_MaskColor
                    Else
                        ' Paint a Disabled Icon
                        cPaint.PaintDisabledStdPic UserControl.hdc, lPosX, lPosY, lBtnWidth, _
                            lBtnHeight, .Image, 0, 0, m_MaskColor
                    End If
            
                Case ezCheck
                    If .Enabled Then
                        cPaint.PaintTransparentStdPic UserControl.hdc, lPosX, lPosY, lBtnWidth, _
                            lBtnHeight, .Image, 0, 0, m_MaskColor
                    Else
                        ' Paint a Disabled Icon
                        cPaint.PaintDisabledStdPic UserControl.hdc, lPosX, lPosY, lBtnWidth, _
                            lBtnHeight, .Image, 0, 0, m_MaskColor
                    End If
                
                Case ezSeparator
                    ' Draw A Separator
                    lBtnWidth = 4
                    Select Case UserControl.Extender.Align
                        Case vbAlignNone, vbAlignTop, vbAlignBottom
                            DrawLine lPosX, , ezFrameSunken
                            .Height = UserControl.ScaleHeight - 2
                            .Width = 4
                        Case vbAlignLeft, vbAlignRight
                            DrawLine lPosY, , ezFrameSunken
                            .Height = 4
                            .Width = UserControl.ScaleWidth - 2
                    End Select
    
                Case ezPlaceholder
                
                Case ezComboBox
                    ' Draw the Main Button
                    If .Enabled Then
                        cPaint.PaintTransparentStdPic UserControl.hdc, lPosX, lPosY, lBtnWidth, _
                            lBtnHeight, .Image, 0, 0, m_MaskColor
                    Else
                        ' Paint a Disabled Icon
                        cPaint.PaintDisabledStdPic UserControl.hdc, lPosX, lPosY, lBtnWidth, _
                            lBtnHeight, .Image, 0, 0, m_MaskColor
                    End If
                    
                    ' Change the Width/Height so that we can draw the Drop Arrow
                    Select Case UserControl.Extender.Align
                        Case vbAlignNone, vbAlignTop, vbAlignBottom
                            lPosXcbo = lPosX + lBtnWidth + 4    ' X Position to draw the drop button
                            lPosYcbo = lPosY + 4                ' Y Position to draw the drop button
                            lPosX = lPosX + lCboButtonSize + 4                  ' Change the next Pos by 4 for the drop button
                            
                        Case vbAlignLeft, vbAlignRight
                            lPosYcbo = lPosY + lBtnHeight + 4   ' X Position to draw the drop button
                            lPosXcbo = lPosX + 4                ' Y Position to draw the drop button
                            lPosY = lPosY + lCboButtonSize + 4                 ' Change the next Pos by 4 for the drop button
                    End Select
                    
                    ' Draw the Drop Arrow
                    cPaint.PaintTransparentStdPic UserControl.hdc, lPosXcbo, lPosYcbo, 7, 7, _
                        LoadResPicture("IDB_COMBOBUTTON", vbResBitmap), 0, 0, m_MaskColor
                    ' Add the Combo BOx Drop down buton to the width
                    .Width = .Width + 10
            End Select
        End With
        
        ' Check the align property to see what Coordinate to increment
        Select Case UserControl.Extender.Align
            Case vbAlignNone, vbAlignTop, vbAlignBottom
            
                lPosX = lPosX + lBtnWidth + 6
        
            Case vbAlignLeft, vbAlignRight
                lPosY = lPosY + lBtnHeight + 6
        End Select
        
    Next iCurrentButton
    
    ' Clean up our unneeded Paint Class
    Set cPaint = Nothing
    
End Sub


Private Sub UserControl_Initialize()
    Dim lngHwnd     As Long
    
    ' The following lines are for the inverted rectangle
    ' Get the handle to the Desktop Window
    lngHwnd = GetDesktopWindow()
    ' The the Device context for that window
    lngScreenDC = GetWindowDC(lngHwnd)
    
    ' Save these values
    mlngTwipsX = Screen.TwipsPerPixelX
    mlngTwipsY = Screen.TwipsPerPixelY
End Sub

' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
'  MouseDown    - Tracks the MouseDown Events
'   Author      - Joseph J Guadagno
' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#

Private Sub UserControl_MouseDown(Button As Integer, Shift As Integer, x As Single, y As Single)
    
    Dim lCurrentButton  As Long     ' The Current Button we are over
    Dim udtMousePos     As POINTAPI ' The Mouse location
    Dim lngReturn       As Long     ' The return value of the API
    
    ' Tell the rest of the system the mouse button is down
    bMouseDown = True
    
    ' What button is being pressed
    lCurrentButton = ButtonAtLocation(x, y)
    
    ' Tell the rest of the system which mouse button is down
    lMouseDownOn = lCurrentButton
    
    ' If the mouse is down over button Zero we will move the toolbar
    ' otherwise we need to show a pressed button.
    If lCurrentButton = 0 Then
        ' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
        '   Move the Toolbar Location
        ' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
        mlngXOffset = x / mlngTwipsX
        mlngYOffset = y / mlngTwipsY
        
    Else
        ' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
        '   Draw the Button Down Effect
        ' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
        With Buttons.Item(lCurrentButton)
            ' Draw the button down effect only if the button style is
            ' non-ezSeparator and the button is enabled
            If .Enabled = True Then
                Select Case .Style
                    Case ezSeparator
                        ' Don't do anything
                        
                    Case ezComboBox
                        ' If the button is a ComboBox we need to handle the drawing
                        ' a little bit differently

                        ' Look to see if we are over the portion
                        If x >= Buttons.Item(lCurrentButton).Left And _
                            x <= (Buttons.Item(lCurrentButton).Left + Buttons.Item(lCurrentButton).Width) - lCboButtonSize And _
                            y >= Buttons.Item(lCurrentButton).Top And _
                            y <= (Buttons.Item(lCurrentButton).Top + Buttons.Item(lCurrentButton).Height) Then
                        
                            ' The Graphic Portion
                            DrawFrame lCurrentButton, ezFrameSunken, ezPortionImage
                            DrawFrame lCurrentButton, ezFrameRaised, ezPortionButton
                        Else
                            ' The Button Portion
                            DrawFrame lCurrentButton, ezFrameSunken, ezPortionButton
                            DrawFrame lCurrentButton, ezFrameRaised, ezPortionImage
                        End If
                    Case Else
                        ' Draw the Sunken Look
                        DrawFrame lCurrentButton, ezFrameSunken
                End Select
            End If
        End With
    End If
    
    ' Send the MouseDown Event to the Container
    RaiseEvent MouseDown(Button, Shift, x, y)
End Sub


' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
'   MouseMove   - Creates the HotTracking effect that is common
'       in most Office97 Toolbars
'   Author      - Joseph J Guadagno
' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
Private Sub UserControl_MouseMove(Button As Integer, Shift As Integer, x As Single, y As Single)
    
    Static lLastButton  As Long     ' The Last button we were over
    Dim lCurrentButton  As Long
    Dim udtMousePos     As POINTAPI ' The Mouse location
    Dim lngReturn       As Long     ' The return value of the API
        
    ' What button is the mouse located over
    lCurrentButton = ButtonAtLocation(x, y)
    
    DoEvents

    ' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
    '   Erase the Last Button
    ' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
    ' If the we have a last button we need to erase it
    If (lLastButton <> lCurrentButton) Then
        If lLastButton = 0 Then
            ' Do nothing
        Else
            With Buttons.Item(lLastButton)
                Select Case .Style
                    Case ezSeparator
                        ' Do Nothing
                    Case ezComboBox
                        With Buttons.Item(lLastButton)
                            Select Case UserControl.Extender.Align
                                Case vbAlignNone, vbAlignTop, vbAlignBottom
                                    DrawLine .Left + .Width - lCboButtonSize, ezHorizontal, ezFrameNone
                                Case vbAlignLeft, vbAlignRight
                                    DrawLine .Left + .Width - lCboButtonSize, ezVertical, ezFrameNone
                            End Select
                        End With
                        DrawFrame lLastButton, ezFrameNone, ezPortionButton
                        DrawFrame lLastButton, ezFrameNone, ezPortionImage
                    Case Else
                        DrawFrame lLastButton, ezFrameNone
                End Select
            End With
        End If
    End If
    
    ' If Mouse Button is down then draw Sunken Button
    If bMouseDown = True Then

        If lCurrentButton = 0 Then
            ' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
            '   Move the Toolbar
            ' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
            ' Get the Current Cursor position
            lngReturn = GetCursorPos(udtMousePos)
            
            If lngReturn Then
                ' Erase the last Rectangle
                
                lngReturn = InvertRect(lngScreenDC, udtRect)
                lngReturn = InvertRect(lngScreenDC, udtInnerRect)
                
                ' Set up the coordinates of the inverted rectangle
                With udtRect
                    .Top = udtMousePos.y - mlngYOffset
                    .Left = udtMousePos.x - mlngXOffset
                    .Bottom = udtMousePos.y + (UserControl.Height / mlngTwipsY) - mlngYOffset
                    .Right = udtMousePos.x + (UserControl.Width / mlngTwipsX) - mlngXOffset
                End With
                ' The inner rectangle
                With udtInnerRect
                    .Top = udtRect.Top + 2
                    .Left = udtRect.Left + 2
                    .Bottom = udtRect.Bottom - 2
                    .Right = udtRect.Right - 2
                End With
                
                ' Draw the new rectangle
                lngReturn = InvertRect(lngScreenDC, udtRect)
                lngReturn = InvertRect(lngScreenDC, udtInnerRect)

            End If
                        
        Else

            ' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
            '   Draw the Sunken Button
            ' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
            With Buttons.Item(lCurrentButton)
                ' Draw the button down effect only if the button style is
                ' non-ezSeparator and the button is enabled
                If .Enabled = True Then
                    Select Case .Style
                        Case ezSeparator
                            ' Don't do anything
                            
                        Case ezComboBox
                            ' If the button is a ComboBox we need to handle the drawing
                            ' a little bit differently
    
                            ' Look to see if we are over the portion
                            If x >= Buttons.Item(lCurrentButton).Left And _
                                x <= (Buttons.Item(lCurrentButton).Left + Buttons.Item(lCurrentButton).Width) - lCboButtonSize And _
                                y >= Buttons.Item(lCurrentButton).Top And _
                                y <= (Buttons.Item(lCurrentButton).Top + Buttons.Item(lCurrentButton).Height) Then
                            
                                ' The Graphic Portion
                                DrawFrame lCurrentButton, ezFrameSunken, ezPortionImage
                                DrawFrame lCurrentButton, ezFrameRaised, ezPortionButton
                            Else
                                ' The Button Portion
                                DrawFrame lCurrentButton, ezFrameSunken, ezPortionButton
                                DrawFrame lCurrentButton, ezFrameRaised, ezPortionImage
                            End If
                        Case Else
                            ' Draw the Sunken Look
                            DrawFrame lCurrentButton, ezFrameSunken
                    End Select
                End If
            End With
            lLastButton = lCurrentButton
        End If
    Else
        ' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
        '   Draw the Raised Button
        ' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
        
        ' We don't need to do anything
        If lCurrentButton = 0 Then
            If UserControl.Extender.ToolTipText <> "" Then
                UserControl.Extender.ToolTipText = ""
            End If
        
            Exit Sub
        End If
        
        ' First we check to see if the Mouse button is Down
        ' if the mouse button down then draw a sunken image
        ' if not draw a raised image
        
        With Buttons.Item(lCurrentButton)
            ' If the Button is a separator don't draw the sunken
            ' effect, if it is a combo box draw the addition line
            Select Case .Style
                Case ezSeparator
                    ' The next time we come in here there will not be a
                    ' button to clear
                    lLastButton = 0
                    ' Reset the tooltip
                    If UserControl.Extender.ToolTipText <> "" Then
                        UserControl.Extender.ToolTipText = ""
                    End If
                Case ezComboBox
                    
                    UserControl.Extender.ToolTipText = .ToolTipText
                    ' Draw the Raised Effect if the Button is Enabled
                    If .Enabled = True Then
                        DrawFrame lCurrentButton, ezFrameRaised, ezPortionImage
                        DrawFrame lCurrentButton, ezFrameRaised, ezPortionButton
                        
                        ' Draw the extra line
                        Select Case UserControl.Extender.Align
                            Case vbAlignNone, vbAlignTop, vbAlignBottom
                                DrawLine .Left + .Width - lCboButtonSize, ezHorizontal, ezFrameSunken
                            Case vbAlignLeft, vbAlignRight
                                DrawLine .Left + .Width - lCboButtonSize, ezVertical, ezFrameSunken
                        End Select
                    End If
                    lLastButton = lCurrentButton
                Case Else
                    
                    UserControl.Extender.ToolTipText = .ToolTipText
                    ' Draw the Raised Effect if the button is enabled
                    If .Enabled = True Then
                        DrawFrame lCurrentButton, ezFrameRaised
                    End If
                    lLastButton = lCurrentButton
            End Select
        
        End With

    End If
    
    ' Send the MouseMove Event
    RaiseEvent MouseMove(Button, Shift, x, y)
End Sub


' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
'  MouseUp      - Tracks the MouseDown Events
'   Author      - Joseph J Guadagno
' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
'
Private Sub UserControl_MouseUp(Button As Integer, Shift As Integer, x As Single, y As Single)
    
    Dim lCurrentButton  As Long
    Dim lngReturn       As Long
    Dim lngCYCaption    As Long
    Dim lngCYdlgFrame   As Long
    Dim lngCXdlgFrame   As Long

    Dim udtFormInPx     As RECT
    
    ' What button are we over
    lCurrentButton = ButtonAtLocation(x, y)
    
    If lCurrentButton <> 0 Then
        If Buttons.Item(lCurrentButton).Enabled = True Then
            RaiseEvent Click(lCurrentButton, Buttons.Item(lCurrentButton).Key)
        End If
    End If
    
    If lCurrentButton = 0 Then
        ' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
        '   Drop the Toolbar
        ' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
        
        ' Erase the previous inverted rectangle
        lngReturn = InvertRect(lngScreenDC, udtRect)
        lngReturn = InvertRect(lngScreenDC, udtInnerRect)
        
        ' Convert the forms demiension to pixels
        With udtFormInPx
            .Left = UserControl.Extender.Parent.Left / mlngTwipsY
            .Top = UserControl.Extender.Parent.Top / mlngTwipsX
            .Bottom = .Top + (UserControl.Extender.Parent.Height / mlngTwipsY)
            .Right = .Left + (UserControl.Extender.Parent.Width / mlngTwipsX)
        End With
        
        
        ' Get the system metrics for the border and caption
        lngCYCaption = GetSystemMetrics(SM_CYCAPTION) * mlngTwipsY
        lngCYdlgFrame = (GetSystemMetrics(SM_CYBORDER) + GetSystemMetrics(SM_CYDLGFRAME)) * mlngTwipsY
        lngCXdlgFrame = (GetSystemMetrics(SM_CXBORDER) + GetSystemMetrics(SM_CXDLGFRAME)) * mlngTwipsX
        
        ' Test to see if the drop cooridinates places the toolbar on the
        '   original container
        If (udtFormInPx.Left < udtRect.Left) And _
            (udtRect.Left < udtFormInPx.Right) And _
            (udtFormInPx.Top < udtRect.Top) And _
            (udtRect.Top < udtFormInPx.Bottom) Then
            
            ' Move the toolbar
            UserControl.Extender.Align = vbAlignNone
            UserControl.Extender.Left = (udtRect.Left * mlngTwipsY) - lngCXdlgFrame
            UserControl.Extender.Top = (udtRect.Top * mlngTwipsX) - lngCYdlgFrame - lngCYCaption
        Else
            
            ' If the toolbar was not dropped on the form then place the
            '   toolbar on the FloatToolbar form
            frmFloatToolbar.Left = udtRect.Left * mlngTwipsY
            frmFloatToolbar.Top = udtRect.Top * mlngTwipsX
            ' Set the height and width now
            frmFloatToolbar.Width = UserControl.Width + (2 * lngCXdlgFrame)
            frmFloatToolbar.Height = UserControl.Height + lngCYdlgFrame + lngCYCaption + 15
            lngReturn = SetParent(UserControl.hwnd, frmFloatToolbar.hwnd)
            UserControl.Extender.Align = vbAlignTop
            frmFloatToolbar.Show
        End If
    Else
        ' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
        '   Erase the Button Drop effect
        ' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#

        ' If the last button was a ComboBox then erase the line
        If Buttons.Item(lCurrentButton).Style = ezComboBox Then
            With Buttons.Item(lCurrentButton)
                Select Case UserControl.Extender.Align
                    Case vbAlignNone, vbAlignTop, vbAlignBottom
                        DrawLine .Left + .Width - lCboButtonSize, ezHorizontal, ezFrameNone
                    Case vbAlignLeft, vbAlignRight
                        DrawLine .Left + .Width - lCboButtonSize, ezVertical, ezFrameNone
                End Select
                DrawFrame lCurrentButton, ezFrameNone, ezPortionImage
                DrawFrame lCurrentButton, ezFrameNone, ezPortionButton
            End With
        End If
        ' Undo the Sunken look
        DrawFrame lCurrentButton, ezFrameNone

    End If
    
    ' Tell the rest of the toolbar that the mouse is not down
    bMouseDown = False
    lMouseDownOn = 0
    
    ' Send the Mouse Up Event
    RaiseEvent MouseUp(Button, Shift, x, y)

End Sub


Private Sub UserControl_Paint()
    DrawToolbar
End Sub
Private Sub UserControl_Resize()
    Select Case UserControl.Extender.Align
        Case vbAlignTop, vbAlignBottom
            UserControl.Height = MAX_HEIGHT
        Case vbAlignLeft, vbAlignRight
            UserControl.Width = MAX_WIDTH
    End Select
    
    DrawToolbar
End Sub
Public Sub Refresh()
Attribute Refresh.VB_Description = "Forces a complete repaint of a object."
     
End Sub

Public Function Customize() As Variant

End Function

Public Function RestoreToolbar() As Variant

End Function

Public Function SaveToolbar() As Variant

End Function
'Initialize Properties for User Control
Private Sub UserControl_InitProperties()
    m_MaskColor = m_def_MaskColor
End Sub

'Load property values from storage
Private Sub UserControl_ReadProperties(PropBag As PropertyBag)
    m_MaskColor = PropBag.ReadProperty("MaskColor", m_def_MaskColor)
End Sub

Private Sub UserControl_Terminate()
    Set Buttons = Nothing
    Set Button = Nothing
End Sub

'Write property values to storage
Private Sub UserControl_WriteProperties(PropBag As PropertyBag)
    Call PropBag.WriteProperty("MaskColor", m_MaskColor, m_def_MaskColor)
End Sub


'WARNING! DO NOT REMOVE OR MODIFY THE FOLLOWING COMMENTED LINES!
'MappingInfo=UserControl,UserControl,-1,Appearance
Public Property Get Appearance() As Integer
Attribute Appearance.VB_Description = "Returns/sets whether or not an object is painted at run time with 3-D effects."
    Appearance = UserControl.Appearance
End Property

Public Property Get MaskColor() As OLE_COLOR
Attribute MaskColor.VB_Description = "Sets/Returns the mask color for button images"
    MaskColor = m_MaskColor
End Property

Public Property Let MaskColor(ByVal New_MaskColor As OLE_COLOR)
    m_MaskColor = New_MaskColor
    PropertyChanged "MaskColor"
End Property
